@@ -18,9 +18,11 @@ parameter types (e.g. Number).
 
 import os.path
 
-from parameterized import Parameterized, Parameter, String, \
+from param.parameterized import Parameterized, Parameter, String, \
      descendents, ParameterizedFunction, ParamOverrides
 
+from param.version import Version
+__version__ = Version(release=(1,2,0), fpath=__file__, commit="$Format:%h$")
 
 #: Top-level object to allow messaging not tied to a particular
 #: Parameterized object, as in 'param.main.warning("Invalid option")'.
@@ -32,31 +34,6 @@ main=Parameterized(name="main")
 random_seed = 42
 
 
-# Set __version__ using versioneer.
-#
-# Encased in a try/except to guarantee this does not interfere with
-# using param (i.e. __init__.py and parameterized.py remain the only
-# two necessary files).
-try:
-    from ._version import get_versions
-    __version__ = get_versions()['version']
-    del get_versions
-    if __version__=='unknown': 
-        # Think 'unknown' is what versioneer returns if unable to
-        # determine version info
-        raise ValueError       
-except:
-    main.warning("""\
-Unable to determine the version of this copy of param.
-
-For an official release, the version is stored in param/_version.py.
-
-For a development copy, the version information is requested from Git
-by param/_version.py.
-""")
-    __version__ = 'unknown'
-
-
 def produce_value(value_obj):
     """
     A helper function that produces an actual parameter from a stored
@@ -230,7 +207,7 @@ class Time(Parameterized):
     def __iter__(self): return self
 
 
-    def next(self):
+    def __next__(self):
         timestep = self.time_type(self.timestep)
 
         if self._exhausted is None:
@@ -242,6 +219,9 @@ class Time(Parameterized):
             raise StopIteration
         return self._time
 
+    # For Python 2 compatibility; can be removed for Python 3.
+    next = __next__
+
     def __call__(self, val=None, time_type=None):
         """
         When called with no arguments, returns the current time value.
@@ -268,6 +248,10 @@ class Time(Parameterized):
         return self._time
 
 
+    def advance(self, val):
+        self += val
+
+
     def __iadd__(self, other):
         self._time = self._time + self.time_type(other)
         return self
@@ -461,11 +445,16 @@ class Dynamic(Parameter):
             return gen
 
 
-# CEBNOTE: isinstance(x,Number) should be possible in Python 2.6
-# (Number is a new abstract base class).
-# http://docs.python.org/whatsnew/2.6.html
-import operator
-_is_number = operator.isNumberType
+import numbers
+def _is_number(obj):
+    if isinstance(obj, numbers.Number): return True
+    # The extra check is for classes that behave like numbers, such as those
+    # found in numpy, gmpy, etc.
+    elif (hasattr(obj, '__int__') and hasattr(obj, '__add__')): return True
+    # This is for older versions of gmpy
+    elif hasattr(obj, 'qdiv'): return True
+    else: return False
+
 
 def identity_hook(obj,val): return val
 
@@ -828,8 +817,8 @@ def concrete_descendents(parentclass):
 
     Only non-abstract classes will be included.
     """
-    return dict([(c.__name__,c) for c in descendents(parentclass)
-                 if not _is_abstract(c)])
+    return dict((c.__name__,c) for c in descendents(parentclass)
+                if not _is_abstract(c))
 
 
 
@@ -982,9 +971,9 @@ class ObjectSelector(Selector):
         # Parameterized instances. Think this is an sf.net bug/feature
         # request. Temporary fix: don't use obj.name if unavailable.
         try:
-            d=dict([(obj.name,obj) for obj in self.objects])
+            d=dict((obj.name,obj) for obj in self.objects)
         except AttributeError:
-            d=dict([(obj,obj) for obj in self.objects])
+            d=dict((obj,obj) for obj in self.objects)
         return d
 
 
@@ -1023,7 +1012,7 @@ class ClassSelector(Selector):
         (see concrete_descendents()).
         """
         classes = concrete_descendents(self.class_)
-        d=dict([(name,class_) for name,class_ in classes.items()])
+        d=dict((name,class_) for name,class_ in classes.items())
         if self.allow_None:
             d['None']=None
         return d
@@ -1256,7 +1245,7 @@ class Path(Parameter):
         """
         try:
             self._resolve(val)
-        except IOError, e:
+        except IOError as e:
             Parameterized(name="%s.%s"%(obj.name,self._attrib_name)).warning('%s'%(e.args[0]))
 
         super(Path,self).__set__(obj,val)
